/*
    This is part of brecord.
    B-record generation routines.
    Copyright (C) 2006  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

static const char rcsid[]="$Id$";


/***************************************************************************/
/* include vari */

#include "hex.h"

/***************************************************************************/
/* define interni alla libreria */

#define RECLEN		4		// numero di bytes per record
#define WRITE		0x00
#define READ		0x20

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

int DataBrecord(unsigned long int Addr,unsigned char *Data,char *Str);
  /* Dato l'indirizzo corrente e un puntatore al data stream crea un
     B-record contenente i dati specificati. La stringa viene creata
     all'interno del buffer Str e viene terminata, in uscita viene
     restituito il numero di bytes prelevati da Data. */

void EntryBrecord(unsigned long int Addr,char *Str);
  /* Crea un B-record contenente il punto di avvio del programma.
     La stringa viene creata nel buffer Str e viene terminata. */
     
int WriteBrecord(unsigned long int Addr,unsigned long int Value,char *Str);
  /* Crea all'interno di Str un Brecord per la scrittura del valore a 32bit
     Value all'indirizzo Addr. In uscita restituisce la lunghezza del
     brecord. */

int ReadBrecord(unsigned long int Addr,char *Str);
  /* Crea all'interno di Str un Brecord per la lettura del valore a 32bit
     che si trova all'indirizzo Addr. In uscita restituisce la lunghezza del
     brecord. */
     
/***************************************************************************/
/* funzioni */

int DataBrecord(unsigned long int Addr,unsigned char *Data,char *Str)
{
  /* Dato l'indirizzo corrente e un puntatore al data stream crea un
     B-record contenente i dati specificati. La stringa viene creata
     all'interno del buffer Str e viene terminata, in uscita viene
     restituito il numero di bytes prelevati da Data. */


     
  char	*Join;				// puntatore al punto di giunta
  int	i;



  Join=LongToHex(Addr,Str);		// inserisce l'indirizzo
  Join=ByteToHex(RECLEN,Join);		// inserisce il numero di bytes
  for(i=0;i<RECLEN;++i)
    Join=ByteToHex(Data[i],Join);	// inserisce i dati
  *Join=0;				// termina la stringa
  return RECLEN;			// termina
}



void EntryBrecord(unsigned long int Addr,char *Str)
{
  /* Crea un B-record contenente il punto di avvio del programma.
     La stringa viene creata nel buffer Str e viene terminata. */
  

  
  char	*Join;				// puntatore al punto di giunta
  

  
  Join=LongToHex(Addr,Str);		// inserisce l'indirizzo
  Join=ByteToHex(0,Join);		// inserisce il numero di bytes
  *Join=0;				// termina la stringa
  return;				// termina
}



int WriteBrecord(unsigned long int Addr,unsigned long int Value,char *Str)
{
  /* Crea all'interno di Str un Brecord per la scrittura del valore a 32bit
     Value all'indirizzo Addr. In uscita restituisce la lunghezza del
     brecord. */
  
  
  
  char	*Join;				// puntatore al punto di giunta


  
  //Str[0]=0x0d;				// inserisce ENTER
  //Join=&Str[1];				// punto di inserimento
  Join=&Str[0];				// punto di inserimento
  Join=LongToHex(Addr,Join);		// inserisce l'indirizzo
  Join=ByteToHex(0xc4|WRITE,Join);	// inserisce il numero di bytes
  Join=LongToHex(Value,Join);		// inserisce il valore
  return 18;				// termina
}



int ReadBrecord(unsigned long int Addr,char *Str)
{
  /* Crea all'interno di Str un Brecord per la lettura del valore a 32bit
     che si trova all'indirizzo Addr. In uscita restituisce la lunghezza del
     brecord. */
  
  
  
  char	*Join;				// puntatore al punto di giunta


  
  //Str[0]=0x0d;				// inserisce ENTER
  //Join=&Str[1];				// punto di inserimento
  Join=&Str[0];				// punto di inserimento
  Join=LongToHex(Addr,Join);		// inserisce l'indirizzo
  Join=ByteToHex(0xc4|READ,Join);	// inserisce il numero di bytes
  return 10;				// termina
}

/* $Source$ */

