/*
    This is part of brecord.
    Hexadecimal conversion routines.
    Copyright (C) 2006  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

static const char rcsid[]="$Id$";


/***************************************************************************/
/* include vari */

/***************************************************************************/
/* define interni alla libreria */

#define UPPERCASE	7
#define LOWERCASE	39
#define LETTER		UPPERCASE		// lettere utilizzate

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

char HighChar(unsigned char Byte);
  /* Dato un valore a 8 bit restituisce la cifra esadecimale che rappresenta
     il valore del nibble alto. */

char LowChar(unsigned char Byte);
  /* Dato un valore a 8 bit restituisce la cifra esadecimale che rappresenta
     il valore del nibble basso. */

char *ByteToHex(unsigned char Byte,char *Str);
  /* Dato in ingresso un valore a 8 bit produce la sua rappresentazione
     in forma esadecimale all'interno di Str. La stringa non viene
     terminata, in uscita viene restituito il puntatore al primo char
     dopo l'ultimo scritto. */
     
char *LongToHex(unsigned long int Long,char *Str);
  /* Dato in ingresso un valore a 32 bit produce la sua rappresentazione
     in forma esadecimale all'interno di Str. La stringa non viene
     terminata, in uscita viene restituito il puntatore al primo char
     dopo l'ultimo scritto. */

/***************************************************************************/
/* funzioni */

char HighChar(unsigned char Byte)
{
  /* Dato un valore a 8 bit restituisce la cifra esadecimale che rappresenta
     il valore del nibble alto. */
  
  
  
  unsigned char	OutValue;		// cifra esadecimale in uscita



  OutValue=(Byte>>4)|0x30;		// lo trasforma in cifra
  if(OutValue>0x39)
    OutValue+=LETTER;			// lo trasforma in lettera
  return((char)OutValue);		// termina
}



char LowChar(unsigned char Byte)
{
  /* Dato un valore a 8 bit restituisce la cifra esadecimale che rappresenta
     il valore del nibble basso. */



  unsigned char	OutValue;		// cifra esadecimale in uscita



  OutValue=(Byte&0x0f)|0x30;		// lo trasforma in cifra
  if(OutValue>0x39)
    OutValue+=LETTER;			// lo trasforma in lettera
  return((char)OutValue);		// termina
}



char *ByteToHex(unsigned char Byte,char *Str)
{
  /* Dato in ingresso un valore a 8 bit produce la sua rappresentazione
     in forma esadecimale all'interno di Str. La stringa non viene
     terminata, in uscita viene restituito il puntatore al primo char
     dopo l'ultimo scritto. */



  Str[0]=HighChar(Byte);		// inserisce la cifra alta
  Str[1]=LowChar(Byte);			// inserisce la cifra bassa
  return(&Str[2]);			// termina
}



char *LongToHex(unsigned long int Long,char *Str)
{
  /* Dato in ingresso un valore a 32 bit produce la sua rappresentazione
     in forma esadecimale all'interno di Str. La stringa non viene
     terminata, in uscita viene restituito il puntatore al primo char
     dopo l'ultimo scritto. */
  
  
  
  Str[0]=HighChar((unsigned char)(Long>>24));	// primo byte
  Str[1]=LowChar((unsigned char)(Long>>24));
  Str[2]=HighChar((unsigned char)(Long>>16));	// secondo byte
  Str[3]=LowChar((unsigned char)(Long>>16));
  Str[4]=HighChar((unsigned char)(Long>>8));	// terzo byte
  Str[5]=LowChar((unsigned char)(Long>>8));
  Str[6]=HighChar((unsigned char)Long);		// quarto byte
  Str[7]=LowChar((unsigned char)Long);
  return(&Str[8]);				// termina
}

/* $Source$ */

