/*
    mxlrevel is a reverse engineering support to investigate the hardware
    of I-Droid01 Brain & Vision module.
    Copyright (C) 2007  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

static const char rcsid[]="$Id$";


/***************************************************************************/
/* include vari */

#include <stdio.h>
#include <unistd.h>
#include "serial.h"
#include "test.h"

/***************************************************************************/
/* define vari */

/* test disponibili */
#define TEST_I2C	0x00000001
#define TEST_CMOS	0x00000002
#define TEST_SPI1	0x00000004
#define TEST_SPI2	0x00000008
#define TEST_MMCSD	0x00000040
#define TEST_UART1	0x00000080
#define TEST_UART2	0x00000100
#define TEST_PORTA	0x00000400
#define TEST_PORTB	0x00000800
#define TEST_PORTC	0x00001000
#define TEST_PORTD	0x00002000
#define TEST_READREG	0x00004000

/***************************************************************************/
/* costanti */

static const char Licenza[]=
	"mxlrevel version 0.03, Copyright (C) 2007  Alessandro Accattini\n";
static const char Help[]=
	"Usage: mxlrelev [OPTIONS]\n"
	"mxlrevel is a reverse engineering tool to investigate the hardware\n"
	"of I-Droid01 Brain & Vision module. It is based on serial dialogue\n"
	"with firmware loader included in mc9328mxl.\n"
	"mxlrevel can drive some I/O pins to allow discovering where pins\n"
	"are routed, it can perform different tests according to the options\n"
	"passed through the command line.\n"
	"Before starting mxlrevel close the programming jumper on the center\n"
	"of the board.\n\n"
	"\t-d DEV\tuse DEV as serial device.\n"
	"\t-h\tdisplays this help.\n"
	"\t-i\ttest the I2C bus.\n"
	"\t-c\ttest the CMOS bus.\n"
	"\t-s\ttest the SPI 1 lines.\n"
	"\t-p\ttest the SPI 2 lines.\n"
	"\t-n\ttest the MMCSD bus.\n"
	"\t-u\ttest the UART 1 lines.\n"
	"\t-w\ttest the UART 2 lines.\n"
	"\t-A\ttest all bits of Port A.\n"
	"\t-B\ttest all bits of Port B.\n"
	"\t-C\ttest all bits of Port C.\n"
	"\t-D\ttest all bits of Port D.\n"
	"\t-R\tread all registers involved in tests.\n"
	"\t-S\tdon't syncronize the firmware loader.\n"
	"Report bugs to bit.trapper@gmail.com\n";

/***************************************************************************/
/* variabili */

/***************************************************************************/
/* prototipi funzioni */

/***************************************************************************/
/* funzioni */



/***************************************************************************/

int main(int argc,char **argv)
{
  char		*Device="/dev/ttyS0";	// serial device
  int		SerialPort;		// fd porta seriale
  int		Opzione;		// per la lettura delle opzioni
  unsigned int	Test=0;			// test to perform
  char		Buffer[4];		// buffer TX/RX
  char		Sync=1;			// sync firmware loader



  /* visualizza la licenza del programma */
  puts(Licenza);
  
  /* legge le opzioni presenti sulla riga di comando */
  while((Opzione=getopt(argc,argv,"d:hicspnuwABCDRS"))!=-1)
  {
    switch(Opzione)
    {
      case 'd':
        Device=optarg;			// imposta il device utilizzato
      break;
      case 'h':
        puts(Help);			// stampa l'help
	return 0;			// termina
      break;
      case 'i':
        Test|=TEST_I2C;			// aggiunge il test
      break;
      case 'c':
        Test|=TEST_CMOS;		// aggiunge il test
      break;
      case 's':
        Test|=TEST_SPI1;		// aggiunge il test
      break;
      case 'p':
        Test|=TEST_SPI2;		// aggiunge il test
      break;
      case 'n':
        Test|=TEST_MMCSD;		// aggiunge il test
      break;
      case 'u':
        Test|=TEST_UART1;		// aggiunge il test
      break;
      case 'w':
        Test|=TEST_UART2;		// aggiunge il test
      break;
      case 'A':
        Test|=TEST_PORTA;		// aggiunge il test
      break;
      case 'B':
        Test|=TEST_PORTB;		// aggiunge il test
      break;
      case 'C':
        Test|=TEST_PORTC;		// aggiunge il test
      break;
      case 'D':
        Test|=TEST_PORTD;		// aggiunge il test
      break;
      case 'R':
        Test|=TEST_READREG;		// aggiunge il test
      break;
      case 'S':
        Sync=0;				// no sync
      break;
      default:
        puts("Unknown option, use -h to see help.");
	return 0;			// termina
      break;
    }
  }

  if(Test==0)
  {
    fputs("No tests selected.\n",stdout);
    return 0;				// termina
  }
 
  /* apre la seriale */
  if((SerialPort=OpenSerialPort(Device,B19200,0,8))<0)
  {
    perror("OpenSerialPort");		// visualizza l'errore
    return 1;				// termina con insuccesso
  }

  /* sincronizza il firmware loader */
  if(Sync!=0)
  {
    fputs("Syncronizing firmware loader... ",stdout);
    Buffer[0]='a';			// prepara il sync char
    write(SerialPort,Buffer,1);		// invia il char
    if((read(SerialPort,Buffer,1)==1)&&(Buffer[0]==':'))
      fputs("done\n",stdout);
    else
    {
      fputs("error",stdout);
      return 1;				// termina con insuccesso
    }
  }
  else
    fputs("Assuming firmware loader is already syncronized.\n",stdout);

  /* ********************************************************************** */
  /* effettua i test */
  if(Test&TEST_I2C)
    TestI2c(SerialPort);		// test bus I2C
  if(Test&TEST_CMOS)
    TestCmos(SerialPort);		// test bus CMOS
  if(Test&TEST_SPI1)
    TestSpi1(SerialPort);		// test SPI 1 lines
  if(Test&TEST_SPI2)
    TestSpi2(SerialPort);		// test SPI 2 lines
  if(Test&TEST_MMCSD)
    TestMmcSd(SerialPort);		// test bus MMCSD
  if(Test&TEST_UART1)
    TestUart1(SerialPort);		// test UART1 lines
  if(Test&TEST_UART2)
    TestUart2(SerialPort);		// test UART2 lines
  if(Test&TEST_PORTA)
    TestPortA(SerialPort);		// test PORT B lines
  if(Test&TEST_PORTB)
    TestPortB(SerialPort);		// test PORT B lines
  if(Test&TEST_PORTC)
    TestPortC(SerialPort);		// test PORT B lines
  if(Test&TEST_PORTD)
    TestPortD(SerialPort);		// test PORT B lines
  if(Test&TEST_READREG)
    TestReadReg(SerialPort);		// read all registers

  fputs("Tests completed.\n",stdout);
  CloseSerialPort(SerialPort);		// chiude la seriale
  return 0;				// termina
}

/* $Source$ */

