/*
    This is part of mxlrever.
    Hardware testing procedures, all the actions performed by mxlrever are
    containde in this file.
    Copyright (C) 2007  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

static const char rcsid[]="$Id$";


/***************************************************************************/
/* include vari */

#include <stdio.h>
#include <unistd.h>
#include "register.h"
#include "mc9328mxl.h"

/***************************************************************************/
/* define interni alla libreria */

//#define CICLES	100
#define CICLES	30
#define PAUSE	20000

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

void GetControl(int SerialPort,char *FnName,unsigned long int BaseAddr,
	char RegLetter,unsigned long int GiusMask,unsigned long int Ocr1Mask,
	unsigned long int Ocr2Mask);
  /* Imposta i registri GIUS_x, ORC1_x, OCR2_x, DDIR_x per prendere il
     controllo delle linee di I/O. */

void MoveLine(int SerialPort,char *FnName,char *BitName,unsigned long int DrReg,
		unsigned long int BitMask,unsigned long int StartValue);
  /* Muove su e giu' ripetutamente la linea indicata. */
  
/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

void TestI2c(int SerialPort);
  /* Tests the I2C bus lines. */

void TestCmos(int SerialPort);
  /* Tests the CMOS camera lines. */

void TestSpi1(int SerialPort);
  /* Tests the SPI 1 lines. */

void TestSpi2(int SerialPort);
  /* Tests the SPI 2 lines. */

void TestMmcSd(int SerialPort);
  /* Tests the MMC/SD bus lines. */

void TestUart1(int SerialPort);
  /* Tests the UART 1 lines. */

void TestUart2(int SerialPort);
  /* Tests the UART 2 lines. */

void TestPortA(int SerialPort);
  /* Tests the PORT A lines. */

void TestPortB(int SerialPort);
  /* Tests the PORT B lines. */

void TestPortC(int SerialPort);
  /* Tests the PORT C lines. */

void TestPortD(int SerialPort);
  /* Tests the PORT D lines. */

void TestReadReg(int SerialPort);
  /* Reads all registers involved in tests. */

/***************************************************************************/
/* funzioni */

void GetControl(int SerialPort,char *FnName,unsigned long int BaseAddr,
	char RegLetter,unsigned long int GiusMask,unsigned long int Ocr1Mask,
	unsigned long int Ocr2Mask)
{
  /* Imposta i registri GIUS_x, ORC1_x, OCR2_x, DDIR_x per prendere il
     controllo delle linee di I/O. */
  
  
  
  unsigned long int	OldValue;			// valore precedente
  unsigned long int	NewValue;			// nuovo valore
  
  
  
  /* imposta il GPIO per accedere alle linee */
  OldValue=ReadRegister(SerialPort,BaseAddr+GIUS_OFF);	// legge il registro
  NewValue=OldValue|GiusMask;				// nuove impostazioni
  printf("%s: modifying GIUS_%c from %8lX to %8lX\n",
  	FnName,RegLetter,OldValue,NewValue);		// print values
  WriteRegister(SerialPort,BaseAddr+GIUS_OFF,NewValue);	// scrive il registro

  OldValue=ReadRegister(SerialPort,BaseAddr+DDIR_OFF);	// legge il registro
  NewValue=OldValue|GiusMask;				// nuove impostazioni
  printf("%s: modifying DDIR_%c from %8lX to %8lX\n",
  	FnName,RegLetter,OldValue,NewValue);		// print values
  WriteRegister(SerialPort,BaseAddr+DDIR_OFF,NewValue);	// scrive il registro
  
  OldValue=ReadRegister(SerialPort,BaseAddr+OCR1_OFF);	// legge il registro
  NewValue=OldValue|Ocr1Mask;				// nuove impostazioni
  printf("%s: modifying OCR1_%c from %8lX to %8lX\n",
  	FnName,RegLetter,OldValue,NewValue);		// print values
  WriteRegister(SerialPort,BaseAddr+OCR1_OFF,NewValue);	// scrive il registro
  
  OldValue=ReadRegister(SerialPort,BaseAddr+OCR2_OFF);	// legge il registro
  NewValue=OldValue|Ocr2Mask;				// nuove impostazioni
  printf("%s: modifying OCR2_%c from %8lX to %8lX\n",
  	FnName,RegLetter,OldValue,NewValue);		// print values
  WriteRegister(SerialPort,BaseAddr+OCR2_OFF,NewValue);	// scrive il registro
  return;						// termina
}



void MoveLine(int SerialPort,char *FnName,char *BitName,unsigned long int DrReg,
		unsigned long int BitMask,unsigned long int StartValue)
{
  /* Muove su e giu' ripetutamente la linea indicata. */
  
  
  
  int			i;
  
  
  
  printf("%s: moving %s up and down\n",FnName,BitName);
  for(i=0;i<CICLES;++i)
  {
    WriteRegister(SerialPort,DrReg,StartValue|BitMask);		// alza
    usleep(PAUSE);						// pausa
    WriteRegister(SerialPort,DrReg,StartValue&~BitMask);	// abbassa
    usleep(PAUSE);						// pausa
  }
}



void TestI2c(int SerialPort)
{
  /* Tests the I2C bus lines. */


  
  unsigned long int	OldValue;			// valore precedente



  printf("%s: starting I2C bus test...\n",__FUNCTION__);
  sleep(1);						// pausa

  /* ********************************************
      I segnali I2C_SDA e I2C_SCL sono mappati
      rispettivamente sui bit 15 e 16 di Port A.
	I2C_SDA  <--> bit 15 Port A
	I2C_SCL  <--> bit 16 Port A
     ******************************************** */

  /* imposta il GPIO per accedere a I2C_SDA e I2C_SCL */
  GetControl(SerialPort,__FUNCTION__,GPIO_A,'A',
  	(1<<I2C_SDA)|(1<<I2C_SCL),
	0x3<<30,
	0x3);						// get lines control

  OldValue=ReadRegister(SerialPort,DR_A);		// legge il registro
  printf("%s: initial value of DR_A: %8lX\n",__FUNCTION__,OldValue);
							
  /* anima le linee */
  sleep(1);							// pausa
  MoveLine(SerialPort,__FUNCTION__,"I2C_SCL",DR_A,1<<I2C_SCL,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"I2C_SDA",DR_A,1<<I2C_SDA,OldValue);
  return;							// termina
}



void TestCmos(int SerialPort)
{
  /* Tests the CMOS camera lines. */


  
  unsigned long int	OldValue;			// valore precedente



  printf("%s: starting CMOS bus test...\n",__FUNCTION__);
  sleep(1);						// pausa

  /* ********************************************
	CSI_MCLK   <--> bit  3 Port A
	CSI_D0     <--> bit  4 Port A
	CSI_D1     <--> bit  5 Port A
	CSI_D2     <--> bit  6 Port A
	CSI_D3     <--> bit  7 Port A
	CSI_D4     <--> bit  8 Port A
	CSI_D5     <--> bit  9 Port A
	CSI_D6     <--> bit 10 Port A
	CSI_D7     <--> bit 11 Port A
	CSI_VSYNC  <--> bit 12 Port A
	CSI_HSYNC  <--> bit 13 Port A
	CSI_PIXCLK <--> bit 14 Port A
     ******************************************** */

  /* imposta il GPIO per accedere alla CMOS interface */
  GetControl(SerialPort,__FUNCTION__,GPIO_A,'A',
  	(1<<CSI_MCLK)|(1<<CSI_D0)|(1<<CSI_D1)|(1<<CSI_D2)|
	(1<<CSI_D3)|(1<<CSI_D4)|(1<<CSI_D5)|(1<<CSI_D6)|
	(1<<CSI_D7)|(1<<CSI_VSYNC)|(1<<CSI_HSYNC)|(1<<CSI_PIXCLK),
	(0x3<<6)|(0x3<<8)|(0x3<<10)|(0x3<<12)|(0x3<<14)|
	(0x3<<16)|(0x3<<18)|(0x3<<20)|(0x3<<22)|(0x3<<24)|
	(0x3<<26)|(0x3<<28),
	0);						// get lines control
  
  OldValue=ReadRegister(SerialPort,DR_A);		// legge il registro
  printf("%s: initial value of DR_A: %8lX\n",__FUNCTION__,OldValue);
							
  /* anima le linee */
  sleep(1);							// pausa
  MoveLine(SerialPort,__FUNCTION__,"CSI_MCLK",DR_A,1<<CSI_MCLK,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"CSI_D0",DR_A,1<<CSI_D0,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"CSI_D1",DR_A,1<<CSI_D1,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"CSI_D2",DR_A,1<<CSI_D2,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"CSI_D3",DR_A,1<<CSI_D3,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"CSI_D4",DR_A,1<<CSI_D4,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"CSI_D5",DR_A,1<<CSI_D5,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"CSI_D6",DR_A,1<<CSI_D6,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"CSI_D7",DR_A,1<<CSI_D7,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"CSI_VSYNC",DR_A,1<<CSI_VSYNC,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"CSI_HSYNC",DR_A,1<<CSI_HSYNC,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"CSI_PIXCLK",DR_A,1<<CSI_PIXCLK,OldValue);
  return;							// termina
}



void TestSpi1(int SerialPort)
{
  /* Tests the SPI 1 lines. */


  
  unsigned long int	OldValue;			// valore precedente



  printf("%s: starting SPI 1 test...\n",__FUNCTION__);
  sleep(1);						// pausa

  /* ********************************************
      Mappatura dei segnali dell'SPI 1:
        SPI1_SPI_RDY  <--> bit 13 port C
	SPI1_SCLK     <--> bit 14 port C
	SPI1_SS       <--> bit 15 port C
	SPI1_MISO     <--> bit 16 port C
	SPI1_MOSI     <--> bit 17 port C
     ******************************************** */

  /* imposta il GPIO per accedere direttamente alle linee */
  GetControl(SerialPort,__FUNCTION__,GPIO_C,'C',
  	(1<<SPI1_SPI_RDY)|(1<<SPI1_SCLK)|(1<<SPI1_SS)|
  	(1<<SPI1_MISO)|(1<<SPI1_MOSI),
	(0x3<<26)|(0x03<<28)|(0x03<<30),
	0x3|(0x03<<2));					// get lines control
  
  OldValue=ReadRegister(SerialPort,DR_C);		// legge il registro
  printf("%s: initial value of DR_C: %8lX\n",__FUNCTION__,OldValue);
							
  /* anima le linee */
  sleep(1);							// pausa
  MoveLine(SerialPort,__FUNCTION__,"SPI1_SPI_RDY",DR_C,1<<SPI1_SPI_RDY,
  				OldValue);
  MoveLine(SerialPort,__FUNCTION__,"SPI1_SCLK",DR_C,1<<SPI1_SCLK,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"SPI1_SS",DR_C,1<<SPI1_SS,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"SPI1_MISO",DR_C,1<<SPI1_MISO,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"SPI1_MOSI",DR_C,1<<SPI1_MOSI,OldValue);
  return;							// termina
}



void TestSpi2(int SerialPort)
{
  /* Tests the SPI 2 lines. */


  
  unsigned long int	OldValue;			// valore precedente



  printf("%s: starting SPI 2 test...\n",__FUNCTION__);
  sleep(1);						// pausa

  /* *****************************************************
      Mappatura dei segnali dell'SPI 2 (master only):
	SPI2_SCLK  <--> bit 7  port D
	SPI2_SS    <--> bit 8  port D
	SPI2_RXD   <--> bit 9  port D
	SPI2_TXD   <--> bit 10 port D
     ***************************************************** */
  
  
  
  /* imposta il GPIO per accedere direttamente alle linee */
  GetControl(SerialPort,__FUNCTION__,GPIO_D,'D',
  	(1<<SPI2_SCLK)|(1<<SPI2_SS)|
  	(1<<SPI2_RXD)|(1<<SPI2_TXD),
	(0x3<<14)|(0x03<<16)|(0x03<<18)|(0x3<<20),
	0);						// get lines control
  
  OldValue=ReadRegister(SerialPort,DR_D);		// legge il registro
  printf("%s: initial value of DR_D: %8lX\n",__FUNCTION__,OldValue);
							
  /* anima le linee */
  sleep(1);							// pausa
  MoveLine(SerialPort,__FUNCTION__,"SPI2_SCLK",DR_D,1<<SPI2_SCLK,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"SPI2_SS",DR_D,1<<SPI2_SS,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"SPI2_RXD",DR_D,1<<SPI2_RXD,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"SPI2_TXD",DR_D,1<<SPI2_TXD,OldValue);
  return;							// termina
}



void TestMmcSd(int SerialPort)
{
  /* Tests the MMC/SD bus lines. */


  
  unsigned long int	OldValue;			// valore precedente



  printf("%s: starting MMCSD test...\n",__FUNCTION__);
  sleep(1);						// pausa

  /* *****************************************************
      Mappatura dei segnali MMCSD:
	SD_DAT_0   <--> bit  8 Port B
	SD_DAT_1   <--> bit  9 Port B
	SD_DAT_2   <--> bit 10 Port B
	SD_DAT_3   <--> bit 11 Port B
	SD_CLK     <--> bit 12 Port B
	SD_CMD     <--> bit 13 Port B
     ***************************************************** */

  /* imposta il GPIO per accedere direttamente alle linee */
  GetControl(SerialPort,__FUNCTION__,GPIO_B,'B',
  	(1<<SD_DAT_0)|(1<<SD_DAT_1)|(SD_DAT_2)|
  	(1<<SD_DAT_3)|(1<<SD_CLK)|(1<<SD_CMD),
	(0x3<<16)|(0x3<<18)|(0x3<<20)|(0x3<<22)|
	(0x3<<24)|(0x3<<26),
	0);						// get lines control
  
  OldValue=ReadRegister(SerialPort,DR_B);		// legge il registro
  printf("%s: initial value of DR_B: %8lX\n",__FUNCTION__,OldValue);
							
  /* anima le linee */
  sleep(1);							// pausa
  MoveLine(SerialPort,__FUNCTION__,"SD_DAT_0",DR_B,1<<SD_DAT_0,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"SD_DAT_1",DR_B,1<<SD_DAT_1,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"SD_DAT_2",DR_B,1<<SD_DAT_2,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"SD_DAT_3",DR_B,1<<SD_DAT_3,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"SD_CLK",DR_B,1<<SD_CLK,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"SD_CMD",DR_B,1<<SD_CMD,OldValue);
  return;							// termina
}



void TestUart1(int SerialPort)
{
  /* Tests the UART 1 lines. */


  
  unsigned long int	OldValue;			// valore precedente



  printf("%s: starting UART 1 test...\n",__FUNCTION__);
  sleep(1);						// pausa

  /* *****************************************************
      Mappatura dei segnali dell'UART 1:
	UART1_RXD  <--> bit 12 Port C
	UART1_TXD  <--> bit 11 Port C
	UART1_RTS  <--> bit 10 Port C
	UART1_CTS  <--> bit 9  Port C
     ***************************************************** */

  /* imposta il GPIO per accedere alla UART1 */
  GetControl(SerialPort,__FUNCTION__,GPIO_C,'C',
  	(1<<UART1_RTS)|(1<<UART1_CTS),
	(0x3<<20)|(0x3<<18),
	0);						// get lines control
  
  OldValue=ReadRegister(SerialPort,DR_C);		// legge il registro
  printf("%s: initial value of DR_C: %8lX\n",__FUNCTION__,OldValue);
							
  /* anima le linee */
  sleep(1);							// pausa
  MoveLine(SerialPort,__FUNCTION__,"UART1_RTS",DR_C,1<<UART1_RTS,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"UART1_CTS",DR_C,1<<UART1_CTS,OldValue);
  return;							// termina
}



void TestUart2(int SerialPort)
{
  /* Tests the UART 2 lines. */


  
  unsigned long int	OldValue;			// valore precedente



  printf("%s: starting UART 2 test...\n",__FUNCTION__);
  sleep(1);						// pausa

  /* *****************************************************
      Mappatura dei segnali dell'UART 2:
	UART2_RXD  <--> bit 31 Port B
	UART2_TXD  <--> bit 30 Port B
	UART2_RTS  <--> bit 29 Port B
	UART2_CTS  <--> bit 28 Port B
     ***************************************************** */

  /* imposta il GPIO per accedere direttamente alle linee */
  GetControl(SerialPort,__FUNCTION__,GPIO_B,'B',
  	(1<<UART2_RXD)|(1<<UART2_TXD)|
	(1<<UART2_RTS)|(1<<UART2_CTS),
	0,
	(0x3<<30)|(0x3<<28)|(0x3<<26)|(0x3<<24));	// get lines control
  
  OldValue=ReadRegister(SerialPort,DR_B);		// legge il registro
  printf("%s: initial value of DR_B: %8lX\n",__FUNCTION__,OldValue);
							
  /* anima le linee */
  sleep(1);							// pausa
  MoveLine(SerialPort,__FUNCTION__,"UART2_RXD",DR_B,1<<UART2_RXD,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"UART2_TXD",DR_B,1<<UART2_TXD,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"UART2_RTS",DR_B,1<<UART2_RTS,OldValue);
  MoveLine(SerialPort,__FUNCTION__,"UART2_CTS",DR_B,1<<UART2_CTS,OldValue);
  return;							// termina
}



void TestPortA(int SerialPort)
{
  /* Tests the PORT A lines. */
  
  
  
  unsigned long int	OldValue;			// valore precedente
  int			i;
  char			PinName[24];



  printf("%s: starting PORT A test...\n",__FUNCTION__);
  sleep(1);						// pausa

  /* *****************************************************
      Mappatura dei segnali della porta B:
        PA2
     ***************************************************** */
  
  /* imposta il GPIO per accedere direttamente alle linee */
  GetControl(SerialPort,__FUNCTION__,GPIO_A,'A',
  	(1<<2),
	(0x3<<4),
	0);						// get lines control
  
  OldValue=ReadRegister(SerialPort,DR_A);		// legge il registro
  printf("%s: initial value of DR_A: %8lX\n",__FUNCTION__,OldValue);
							
  /* anima le linee */
  sleep(1);							// pausa
  printf("%s: moving PORT A lines up and down\n",__FUNCTION__);
  for(i=2;i<=2;++i)
  {
    sprintf(PinName,"PORT A pin %i",i);
    MoveLine(SerialPort,__FUNCTION__,PinName,DR_A,1<<i,OldValue);
  }
  return;							// termina
}



void TestPortB(int SerialPort)
{
  /* Tests the PORT B lines. */
  
  
  
  unsigned long int	OldValue;			// valore precedente
  int			i;
  char			PinName[24];



  printf("%s: starting PORT B test...\n",__FUNCTION__);
  sleep(1);						// pausa

  /* *****************************************************
      Mappatura dei segnali della porta B:
        PB14
        PB15
        PB16
        PB17
        PB18
        PB19
     ***************************************************** */
  
  /* imposta il GPIO per accedere direttamente alle linee */
  GetControl(SerialPort,__FUNCTION__,GPIO_B,'B',
  	(1<<14)|(1<<15)|(1<<16)|(1<<17)|(1<<18)|(1<<19),
	(0x3<<28)|(0x3<<30),
	(0x3<<0)|(0x3<<2)|(0x3<<4)|(0x3<<6));		// get lines control
  
  OldValue=ReadRegister(SerialPort,DR_B);		// legge il registro
  printf("%s: initial value of DR_B: %8lX\n",__FUNCTION__,OldValue);
							
  /* anima le linee */
  sleep(1);							// pausa
  printf("%s: moving PORT B lines up and down\n",__FUNCTION__);
  for(i=14;i<=19;++i)
  {
    sprintf(PinName,"PORT B pin %i",i);
    MoveLine(SerialPort,__FUNCTION__,PinName,DR_B,1<<i,OldValue);
  }
  return;							// termina
}



void TestPortC(int SerialPort)
{
  /* Tests the PORT C lines. */
  
  
  
  unsigned long int	OldValue;			// valore precedente
  int			i;
  char			PinName[24];



  printf("%s: starting PORT C test...\n",__FUNCTION__);
  sleep(1);						// pausa

  /* *****************************************************
      Mappatura dei segnali della porta C:
        PC3
        PC4
        PC5
        PC6
        PC7
        PC8
     ***************************************************** */
  
  /* imposta il GPIO per accedere direttamente alle linee */
  GetControl(SerialPort,__FUNCTION__,GPIO_C,'C',
  	(1<<3)|(1<<4)|(1<<5)|(1<<6)|(1<<7)|(1<<8),
	(0x3<<6)|(0x3<<8)|(0x3<<10)|(0x3<<12)|(0x3<<14)|(0x3<<16),
	0);						// get lines control
  
  OldValue=ReadRegister(SerialPort,DR_C);		// legge il registro
  printf("%s: initial value of DR_C: %8lX\n",__FUNCTION__,OldValue);
							
  /* anima le linee */
  sleep(1);							// pausa
  printf("%s: moving PORT C lines up and down\n",__FUNCTION__);
  for(i=3;i<=8;++i)
  {
    sprintf(PinName,"PORT C pin %i",i);
    MoveLine(SerialPort,__FUNCTION__,PinName,DR_C,1<<i,OldValue);
  }
  return;							// termina
}



void TestPortD(int SerialPort)
{
  /* Tests the PORT D lines. */
  
  
  
  unsigned long int	OldValue;			// valore precedente
  int			i;
  char			PinName[24];



  printf("%s: starting PORT D test...\n",__FUNCTION__);
  sleep(1);						// pausa

  /* *****************************************************
      Mappatura dei segnali della porta D:
        PD7
        PD8
	PD9
	PD10
     ***************************************************** */
  
  /* imposta il GPIO per accedere direttamente alle linee */
  GetControl(SerialPort,__FUNCTION__,GPIO_D,'D',
  	(1<<7)|(1<<8)|(1<<9)|(1<<10),
	(0x3<<14)|(0x3<<16)|(0x3<<18)|(0x3<<20),
	0);						// get lines control
  
  OldValue=ReadRegister(SerialPort,DR_D);		// legge il registro
  printf("%s: initial value of DR_D: %8lX\n",__FUNCTION__,OldValue);
							
  /* anima le linee */
  sleep(1);							// pausa
  printf("%s: moving PORT D lines up and down\n",__FUNCTION__);
  for(i=7;i<=10;++i)
  {
    sprintf(PinName,"PORT D pin %i",i);
    MoveLine(SerialPort,__FUNCTION__,PinName,DR_D,1<<i,OldValue);
  }
  return;							// termina
}



void TestReadReg(int SerialPort)
{
  /* Reads all registers involved in tests. */
  
  
  
  /* legge lo stato dei registri della PORTA A */
  printf("%s: reading PORT A register status...\n",__FUNCTION__);
  printf("%s: GIUS_A status (%8lX) --> %8lX\n",__FUNCTION__,
  		GIUS_A,ReadRegister(SerialPort,GIUS_A));	// print value
  printf("%s: GPR_A  status (%8lX) --> %8lX\n",__FUNCTION__,
  		GPR_A,ReadRegister(SerialPort,GPR_A));		// print value
  printf("%s: DDIR_A status (%8lX) --> %8lX\n",__FUNCTION__,
  		DDIR_A,ReadRegister(SerialPort,DDIR_A));	// print value
  printf("%s: OCR1_A status (%8lX) --> %8lX\n",__FUNCTION__,
  		OCR1_A,ReadRegister(SerialPort,OCR1_A));	// print value
  printf("%s: OCR2_A status (%8lX) --> %8lX\n",__FUNCTION__,
  		OCR2_A,ReadRegister(SerialPort,OCR2_A));	// print value
  printf("%s: DR_A   status (%8lX) --> %8lX\n",__FUNCTION__,
  		DR_A,ReadRegister(SerialPort,DR_A));		// print value

  /* legge lo stato dei registri della PORTA B */
  printf("%s: reading PORT B register status...\n",__FUNCTION__);
  printf("%s: GIUS_B status (%8lX) --> %8lX\n",__FUNCTION__,
  		GIUS_B,ReadRegister(SerialPort,GIUS_B));	// print value
  printf("%s: GPR_B  status (%8lX) --> %8lX\n",__FUNCTION__,
  		GPR_B,ReadRegister(SerialPort,GPR_B));		// print value
  printf("%s: DDIR_B status (%8lX) --> %8lX\n",__FUNCTION__,
  		DDIR_B,ReadRegister(SerialPort,DDIR_B));	// print value
  printf("%s: OCR1_B status (%8lX) --> %8lX\n",__FUNCTION__,
  		OCR1_B,ReadRegister(SerialPort,OCR1_B));	// print value
  printf("%s: OCR2_B status (%8lX) --> %8lX\n",__FUNCTION__,
  		OCR2_B,ReadRegister(SerialPort,OCR2_B));	// print value
  printf("%s: DR_B   status (%8lX) --> %8lX\n",__FUNCTION__,
  		DR_B,ReadRegister(SerialPort,DR_B));		// print value

  /* legge lo stato dei registri della PORTA C */
  printf("%s: reading PORT C register status...\n",__FUNCTION__);
  printf("%s: GIUS_C status (%8lX) --> %8lX\n",__FUNCTION__,
  		GIUS_C,ReadRegister(SerialPort,GIUS_C));	// print value
  printf("%s: GPR_C  status (%8lX) --> %8lX\n",__FUNCTION__,
  		GPR_C,ReadRegister(SerialPort,GPR_C));		// print value
  printf("%s: DDIR_C status (%8lX) --> %8lX\n",__FUNCTION__,
  		DDIR_C,ReadRegister(SerialPort,DDIR_C));	// print value
  printf("%s: OCR1_C status (%8lX) --> %8lX\n",__FUNCTION__,
  		OCR1_C,ReadRegister(SerialPort,OCR1_C));	// print value
  printf("%s: OCR2_C status (%8lX) --> %8lX\n",__FUNCTION__,
  		OCR2_C,ReadRegister(SerialPort,OCR2_C));	// print value
  printf("%s: DR_C   status (%8lX) --> %8lX\n",__FUNCTION__,
  		DR_C,ReadRegister(SerialPort,DR_C));		// print value

  /* legge lo stato dei registri della PORTA D */
  printf("%s: reading PORT D register status...\n",__FUNCTION__);
  printf("%s: GIUS_D status (%8lX) --> %8lX\n",__FUNCTION__,
  		GIUS_D,ReadRegister(SerialPort,GIUS_D));	// print value
  printf("%s: GPR_D  status (%8lX) --> %8lX\n",__FUNCTION__,
  		GPR_D,ReadRegister(SerialPort,GPR_D));		// print value
  printf("%s: DDIR_D status (%8lX) --> %8lX\n",__FUNCTION__,
  		DDIR_D,ReadRegister(SerialPort,DDIR_D));	// print value
  printf("%s: OCR1_D status (%8lX) --> %8lX\n",__FUNCTION__,
  		OCR1_D,ReadRegister(SerialPort,OCR1_D));	// print value
  printf("%s: OCR2_D status (%8lX) --> %8lX\n",__FUNCTION__,
  		OCR2_D,ReadRegister(SerialPort,OCR2_D));	// print value
  printf("%s: DR_D   status (%8lX) --> %8lX\n",__FUNCTION__,
  		DR_D,ReadRegister(SerialPort,DR_D));		// print value
  return;						// termina
}

/* $Source$ */
